@@ -23,16 +23,19 @@ import logging
 import time
 from datetime import datetime
 import os
-
+import tempfile
 import gobject
+import gconf
+import gio
 
+from sugar import env
 from sugar.datastore import dbus_helpers
 from sugar import mime
 
+
 class DSMetadata(gobject.GObject):
     __gsignals__ = {
-        'updated': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
-                    ([]))
+        'updated': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ([])),
     }
 
     def __init__(self, props=None):
@@ -41,7 +44,7 @@ class DSMetadata(gobject.GObject):
             self._props = {}
         else:
             self._props = props
-        
+
         default_keys = ['activity', 'activity_id',
                         'mime_type', 'title_set_by_user']
         for key in default_keys:
@@ -61,13 +64,13 @@ class DSMetadata(gobject.GObject):
 
     def __contains__(self, key):
         return self._props.__contains__(key)
-    
+
     def has_key(self, key):
         return self._props.has_key(key)
 
     def keys(self):
         return self._props.keys()
-    
+
     def get_dictionary(self):
         return self._props
 
@@ -80,7 +83,9 @@ class DSMetadata(gobject.GObject):
         else:
             return default
 
+
 class DSObject(object):
+
     def __init__(self, object_id, metadata=None, file_path=None):
         self.object_id = object_id
         self._metadata = metadata
@@ -93,7 +98,7 @@ class DSObject(object):
             metadata = DSMetadata(dbus_helpers.get_properties(self.object_id))
             self._metadata = metadata
         return self._metadata
-    
+
     def set_metadata(self, metadata):
         if self._metadata != metadata:
             self._metadata = metadata
@@ -105,7 +110,7 @@ class DSObject(object):
             self.set_file_path(dbus_helpers.get_filename(self.object_id))
             self._owns_file = True
         return self._file_path
-    
+
     def set_file_path(self, file_path):
         if self._file_path != file_path:
             if self._file_path and self._owns_file:
@@ -129,27 +134,89 @@ class DSObject(object):
 
     def __del__(self):
         if not self._destroyed:
-            logging.warning('DSObject was deleted without cleaning up first. ' \
+            logging.warning('DSObject was deleted without cleaning up first. '
                             'Call DSObject.destroy() before disposing it.')
             self.destroy()
 
     def copy(self):
         return DSObject(None, self._metadata.copy(), self._file_path)
 
+class RawObject(object):
+
+    def __init__(self, file_path):
+        stat = os.stat(file_path)
+        client = gconf.client_get_default()
+        metadata = {
+                'uid': file_path,
+                'title': os.path.basename(file_path),
+                'timestamp': stat.st_mtime,
+                'mime_type': gio.content_type_guess(filename=file_path),
+                'activity': '',
+                'activity_id': '',
+                'icon-color': client.get_string('/desktop/sugar/user/color'),
+                'description': file_path,
+                }
+
+        self.object_id = file_path
+        self._metadata = DSMetadata(metadata)
+        self._file_path = None
+        self._destroyed = False
+
+    def get_metadata(self):
+        return self._metadata
+
+    metadata = property(get_metadata)
+
+    def get_file_path(self, fetch=True):
+        # we have to create symlink since its a common practice
+        # to create hardlinks to jobject files
+        # and w/o this, it wouldn't work since we have file from mounted device
+        if self._file_path is None:
+            self._file_path = tempfile.mktemp(
+                    prefix='rawobject',
+                    dir=os.path.join(env.get_profile_path(), 'data'))
+            os.symlink(self.object_id, self._file_path)
+        return self._file_path
+
+    file_path = property(get_file_path)
+
+    def destroy(self):
+        if self._destroyed:
+            logging.warning('This RawObject has already been destroyed!.')
+            return
+        self._destroyed = True
+        if self._file_path is not None:
+            if os.path.exists(self._file_path):
+                os.remove(self._file_path)
+            self._file_path = None
+
+    def __del__(self):
+        if not self._destroyed:
+            logging.warning('RawObject was deleted without cleaning up. '
+                            'Call RawObject.destroy() before disposing it.')
+            self.destroy()
+
+
 def get(object_id):
     logging.debug('datastore.get')
+
+    if object_id.startswith('/'):
+        return RawObject(object_id)
+
     metadata = dbus_helpers.get_properties(object_id)
 
     ds_object = DSObject(object_id, DSMetadata(metadata), None)
     # TODO: register the object for updates
     return ds_object
 
+
 def create():
     metadata = DSMetadata()
     metadata['mtime'] = datetime.now().isoformat()
     metadata['timestamp'] = int(time.time())
     return DSObject(object_id=None, metadata=metadata, file_path=None)
 
+
 def write(ds_object, update_mtime=True, transfer_ownership=False,
           reply_handler=None, error_handler=None, timeout=-1):
     logging.debug('datastore.write')
@@ -183,12 +250,14 @@ def write(ds_object, update_mtime=True, transfer_ownership=False,
                                                   transfer_ownership)
         ds_object.metadata['uid'] = ds_object.object_id
         # TODO: register the object for updates
-    logging.debug('Written object %s to the datastore.' % ds_object.object_id)
+    logging.debug('Written object %s to the datastore.', ds_object.object_id)
+
 
 def delete(object_id):
     logging.debug('datastore.delete')
     dbus_helpers.delete(object_id)
 
+
 def find(query, sorting=None, limit=None, offset=None, properties=None,
          reply_handler=None, error_handler=None):
 
@@ -203,10 +272,10 @@ def find(query, sorting=None, limit=None, offset=None, properties=None,
         query['limit'] = limit
     if offset:
         query['offset'] = offset
-    
+
     props_list, total_count = dbus_helpers.find(query, properties,
                                                 reply_handler, error_handler)
-    
+
     objects = []
     for props in props_list:
         object_id = props['uid']
@@ -217,6 +286,7 @@ def find(query, sorting=None, limit=None, offset=None, properties=None,
 
     return objects, total_count
 
+
 def copy(jobject, mount_point):
 
     new_jobject = jobject.copy()
@@ -238,17 +308,22 @@ def copy(jobject, mount_point):
 
     write(new_jobject)
 
+
 def mount(uri, options, timeout=-1):
     return dbus_helpers.mount(uri, options, timeout=timeout)
 
+
 def unmount(mount_point_id):
     dbus_helpers.unmount(mount_point_id)
 
+
 def mounts():
     return dbus_helpers.mounts()
 
+
 def complete_indexing():
     return dbus_helpers.complete_indexing()
 
+
 def get_unique_values(key):
     return dbus_helpers.get_unique_values(key)
